# from ..influx_alert.influx_alert import InfluxAlert

from influx_alert.influx_alert import InfluxAlert


influx_alert = InfluxAlert(influx_auth='', 
                           webhook_url='', 
                           mongodb_auth='')

influx_alert.ping.unreachable()



#!/usr/bin/env python3

import re
import json
import datetime
import time
import uuid
import pytz

import pymongo
import requests
from nb_log import get_logger

from ..endpoint import Endpoint


log = get_logger('extensions')

HEADERS_JSON = {'Content-Type': 'application/json'}

class ExtensionsEndpoint(Endpoint):
    
    # def __init__(self):
    #     self.ALARM_NAME_PING_UNREACHABLE = 'Ping 不可达'
    #     self.ALARM_NAME_BFD_8 = 'BFD状态异常'
    #     self.ALARM_NAME_PORT_DOWN = '端口状态异常'
    #     self.ALARM_NAME_TRAFFIC = '流量告警'
    #     self.ALARM_BGP_STATUS = 'BGP状态异常'
    #     self.ALARM_NO_SNMP_DATA = '无SNMP监控数据'
    #     self.ALARM_NAME_UPS_STATUS = 'UPS状态异常'
        
    #     # upsAdvBatteryReplaceIndicator
    #     self.ALARM_NAME_upsAdvBatteryReplaceIndicator = 'UPS电池需要更换'

    #     self.EVENT_TYPE_TRIGGER = 'trigger'
    #     self.EVENT_TYPE_RESOLVED = 'resolved'

    #     self.PRIORITY_DISASTER = 'disaster'
    #     self.PRIORITY_HIGH = 'high'
    #     self.PRIORITY_WARNING = 'warning'
        
    #     self.PING_LIMIT = 8
    #     self.PING_RESOLVED = 5
    #     self.BFD_LIMIT = 8

    def time_convert_timeobj_to_str(self, timeobj: str=None, timezone_offset: int=8):
        try:
            time_obj_with_offset = timeobj + datetime.timedelta(hours=timezone_offset)
            return time_obj_with_offset.strftime("%Y-%m-%d %H:%M")
        except TypeError:
            log.error(f'时间对象转换失败, 传入的时间对象: [{timeobj}], 时间偏移: [{timezone_offset}]')
            return '时间转换失败'

    @staticmethod
    def convert_timeobj_to_str_date(timeobj):
        return timeobj.strftime("%Y-%m-%d")

    @staticmethod
    def get_now_date():
        '''
        _summary_

        Returns:
            _type_: _description_
        '''
        now = datetime.datetime.now()
        return now.strftime("%Y%m%d")
    
    @staticmethod
    def get_start_and_end_time(last_minute):

        # 获取当前时间戳（精度为秒）
        current_timestamp = int(time.time())

        # 将当前时间戳转换为毫秒级精度
        current_timestamp_ms = current_timestamp * 1000

        # 计算1分钟前的时间戳（精度为秒）
        start_timestamp = current_timestamp - last_minute * 60

        # 将1分钟前的时间戳转换为毫秒级精度
        start_timestamp_ms = start_timestamp * 1000

        # 打印结果
        return start_timestamp_ms, current_timestamp_ms
    
    @staticmethod
    def get_start_and_end_time_tz(last_minute):
        '''
        _summary_

        Args:
            offset (int): 输入秒数

        Returns:
            _type_: _description_
        '''
        
        current_time = datetime.datetime.utcnow() - datetime.timedelta(seconds= last_minute * 60)
        end_time = current_time.strftime("%Y-%m-%dT%H:%M:%SZ")
        start_time = datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ")
        return end_time, start_time

    @staticmethod
    def convert_time(input_time):
        '''
        将时间对象转换为字符串

        Args:
            input_time (_type_): _description_

        Returns:
            _type_: _description_
        '''
        # 输入的时间字符串
        # input_time = "2024-04-17T23:21:35.362383Z"

        # 转换为 datetime 对象
        dt = datetime.datetime.strptime(input_time, "%Y-%m-%dT%H:%M:%S.%fZ")

        # 设置时区为 GMT+8
        timezone = pytz.timezone('Asia/Shanghai')
        dt = dt.replace(tzinfo=pytz.utc).astimezone(timezone)

        # 格式化为所需的字符串格式
        output_time = dt.strftime("%Y-%m-%d %H:%M:%S")

        return output_time

    @staticmethod
    def get_now_time(now: bool=True):
        current_time = datetime.datetime.now()  # 获取当前时间并加上8小时（对应+08:00时区）
        time_string = current_time.strftime("%Y-%m-%dT%H:%M:%S+08:00")  # 格式化时间字符串
        return time_string
    
    @staticmethod
    def get_now_timestamp():
        # current_time = time.time()
        # return int(datetime.datetime.now().timestamp())
        # microseconds = datetime.datetime.fromtimestamp(time.time()).microsecond
        return int(time.time() * 1000)
    
    def time_get_now_time_mongo(self):
        '''
        获取当前时间, 加入了时区信息, 简单是存储在 Mongo 中时格式为 ISODate

        Returns:
            current_time(class): 时间, 格式: 2024-04-23 16:48:11.591589+08:00
        '''
        current_time = datetime.datetime.now(pytz.timezone('Asia/Shanghai'))
        return current_time
    
    @staticmethod
    def convert_time_influx_to_tz(input_time):
        '''
        _summary_

        Args:
            input_time (_type_): _description_

        Returns:
            _type_: _description_
        '''
        # 转换为 datetime 对象
        dt = datetime.datetime.strptime(input_time, "%Y-%m-%dT%H:%M:%SZ")

        # 设置时区为 GMT+8
        timezone = pytz.timezone('Asia/Shanghai')
        dt = dt.replace(tzinfo=pytz.utc).astimezone(timezone)

        # 格式化为所需的字符串格式
        output_time = dt.strftime("%Y-%m-%dT%H:%M:%S+08:00")

        return output_time

    @staticmethod
    def convert_time_influx_to_mongo(input_time):
        '''
        将 influxdb 中的时间转换为 mongo 的格式, 输入是 CMT+0 时区, 输出也是 GMT+0

        Args:
            input_time (_type_): TZ 格式, 时区是 GMT+0

        Returns:
            _type_: _description_
        '''
        # 转换为 datetime 对象
        if '.' in input_time:
            return datetime.datetime.strptime(input_time, "%Y-%m-%dT%H:%M:%S.%fZ")
        else:
            return datetime.datetime.strptime(input_time, "%Y-%m-%dT%H:%M:%SZ")

    @staticmethod
    def convert_str_to_obj(time_str):
        # date_string = '2024-06-01 09:00'
        timezone = pytz.timezone('Asia/Shanghai')

        # 将日期字符串转换为时间对象
        date_object = datetime.datetime.strptime(time_str, '%Y-%m-%d %H:%M')

        # 设置时区
        date_object = timezone.localize(date_object)
        return date_object

    def time_diff_influx_now(self, source_time):
        '''
        计算influxdb默认的时间和当前时间的差异

        Args:
            source_time (_type_): _description_

        Returns:
            _type_: _description_
        '''
        current_time = datetime.datetime.utcnow()
        target_time = datetime.datetime.strptime(source_time, '%Y-%m-%dT%H:%M:%SZ')
        time_diff = current_time - target_time
        minutes_diff = int(time_diff.total_seconds() // 60)
        return minutes_diff


    
    def tool_check_insert_send_mongo(self,
                                    restore_influx: str=None,
                                    url: str=None,
                                    alarm_content: str=None,
                                    priority: str=None,
                                    event_id: str=str(uuid.uuid4()),
                                    alarm_name: str=None,
                                    entity_name: str=None,
                                    event_type: str='trigger',
                                    automate_ts: str='暂无',
                                    suggestion: str='',
                                    referce_name: str='',
                                    referce_link: str='',
                                    alarm_time: str='',
                                    mongo_id: str=None,
                                    is_mongo: bool=True,
                                    is_notify: bool=True,
                                    mail_subject: str=None,
                                    mail_content: str=None):
            '''
            这个函数非常重要!!!
            将发送的告警或恢复事件写入到 InfluxDB, 并发送给 OneAlert

            Args:
                restore_influx (str): _description_
                url (str): _description_
                alarm_content (str): _description_
                priority (str): _description_
                alarm_name (str): _description_
                entity_name (str): _description_
                event_type (str): _description_
                automate_ts (str): _description_
                suggestion (str): _description_
                referce_name (str): _description_
                referce_link (str): _description_
                debug (bool): 如果开启, 告警不插入数据库也不发送到 Onealert
            '''
            if event_type == 'trigger':
                event_id = str(uuid.uuid4())
                feishu_card_color = 'red'

            # 如果为 True, 表示需要插入告警
            if self.mongo_check_alarm_exist(event_type=event_type, alarm_content=alarm_content):
                log.info(f'发现告警: [{alarm_content}], event_id: [{event_id}]')
                if is_mongo:
                    if event_type == 'trigger':
                        resp = self.mongo_insert_alert(
                            event_id=event_id,
                            alarm_content=alarm_content,
                            alarm_name=alarm_name,
                            priority=priority,
                            entity_name=entity_name,
                            restore_influx=restore_influx,
                            url=url,
                            alarm_time=alarm_time)

                        if resp:
                            log.info(f'已插入告警, alarm_content: [{alarm_content}]')
                        else:
                            log.error(f'告警插入到数据库时出错, alarm_content: [{alarm_content}]')
                        
                    elif event_type == 'resolved':
                        log.debug(f'发现告警恢复: [{alarm_content}]')
                        self.mongo_update_resolved(doc_id=mongo_id, resolved_time=self.time_get_now_time_mongo())
                        feishu_card_color = 'green'
                        resolved_time = self.time_get_now_time_mongo()
                        
                if is_notify:
                    
                    alert_dict = { 
                        "eventId": event_id, 
                        "eventType": event_type, 
                        "alarmName": alarm_name, 
                        "entityName": entity_name, 
                        "priority": priority, 
                        "alarmContent": alarm_content, 
                        "alarm_time": self.time_convert_timeobj_to_str(timeobj=alarm_time, timezone_offset=0),
                        "details": '自动排查' + '\n' + f'{automate_ts}' + '\n' + '处置建议: ' + '\n' + suggestion + '\n' + f'参考文档: [{referce_name}]({referce_link})' + '\n'
                    }
                    
                    template_variable= {
                        "color": feishu_card_color,
                        "alarm_time": self.time_convert_timeobj_to_str(timeobj=alarm_time, timezone_offset=0),
                        "alarm_name": alarm_name,
                        "alarm_content": alarm_content,
                        "priority": priority,
                        "details": '该告警出现过: ' + str(self.query_alert_counter(alarm_content)) + '\n' + alert_dict['details']
                    }
                    # log.info(template_variable)
                    log.info(alert_dict)

                    if not self.parent.debug:

                        if self.parent.feishu_app_id:
                            resp  = self.parent.feishu_client.message.send_card(
                                template_id=self.parent.feishu_card_template_id,
                                template_variable= template_variable,
                                receive_id =self.parent.feishu_card_receive_id
                            )
                            log.debug(f'发送飞书卡片消息: [{resp}]')
                        
                        if self.parent.wecom_webhook_url:
                            resp = self.notify_wecom_markdown(alert_dict=alert_dict)
                            log.debug(f'发送企业微信消息: [{resp}]')
                        
                        if self.parent.onealert_webhook_url:
                            log.warning(alert_dict)
                            resp = requests.request(method='POST', url=self.parent.onealert_webhook_url, json=alert_dict, headers=HEADERS_JSON)
                            log.debug(f'发送OneAlert消息,接收到回应: [{resp}]')
                        
                    else:
                        log.debug(alert_dict)

    def mongo_check_alarm_exist(self, event_type, alarm_content) -> bool:
        '''
        _summary_

        Args:
            alarm_content (_type_): 告警内容

        Returns:
            bool: 如果为 True, 表示允许插入告警
        '''
        if event_type == 'trigger':
            query = { "alarm_content": alarm_content }
            fields = {"alarm_name": 1, "alarm_time": 1, "resolved_time": 1}
            result = self.parent.mongo_client.find(query, fields).sort({ "_id": pymongo.DESCENDING }).limit(1)
            
            if self.parent.mongo_client.count_documents(query) == 0:
                # print("没有数据返回，执行插入数据")
                return True
            else:
                for doc in result:
                    if 'resolved_time' in doc:
                        # 当前没有告警, 可以插入数据
                        return True
        elif event_type == 'resolved':
            return True
    def mongo_insert_alert(self,
                     event_id,
                     alarm_content,
                     alarm_name,
                     priority,
                     entity_name,
                     restore_influx,
                     url,
                     alarm_time):
        '''
        插入告警到 MongoDB

        Args:
            event_id (_type_): _description_
            alarm_content (_type_): _description_
            alarm_name (_type_): _description_
            priority (_type_): _description_
            entity_name (_type_): _description_
            restore_influx (_type_): _description_
            url (_type_): _description_
            alarm_time (_type_): _description_
        '''
        data = {
            "event_id": event_id,
            "alarm_name": alarm_name,
            "alarm_content": alarm_content,
            "priority": priority,
            "restore_influx": restore_influx,
            "url": url,
            "alarm_time": alarm_time,
            "entity_name": entity_name
        }
        
        log.debug(f'没有插入过数据, [{data}]')
        resp = self.parent.mongo_client.insert_one(data)
        log.debug(resp)
        return resp.acknowledged

    def mongo_update_resolved(self, doc_id, resolved_time):
        '''
        更新告警为解决

        Args:
            doc_id (_type_): _description_
            resolved_time (_type_): _description_

        Returns:
            _type_: _description_
        '''
        filter_doc = {"_id": doc_id}
        update = {"$set": { "resolved_time": resolved_time}}

        # 执行更新操作
        # return self.collection.update_one(filter_doc, update)
        return self.parent.mongo_client.update_one(filter_doc, update)
    
    def mongo_query_trigger(self, alarm_name: str=None):
        '''
        _summary_

        Returns:
            _type_: _description_
        '''
        if alarm_name:
            query = {
                'alarm_name': alarm_name,
                'resolved_time': {'$exists': False}
            }
        else:
            query = {
                'resolved_time': {'$exists': False}
            } 
        return self.parent.mongo_client.find(query)

    def notify_feishu_markdown(self):
        pass
    
    def notify_feishu_card(self):
        pass
    
    def notify_wecom_markdown(self, alert_dict):
        if alert_dict['eventType'] == 'trigger':
            title = '有新的告警, 请注意!'
            title_color = 'warning'
        else:
            title = '告警恢复!'
            title_color = 'info'
            
        content = f"""
        <font color=\"{title_color}\">{title}</font>\n 
        {alert_dict['alarmContent']} \n
        >告警名称: <font color=\"comment\">{alert_dict['alarmName']}</font> 
        >实例名称: <font color=\"comment\">{alert_dict['entityName']}</font> 
        >优先级: <font color=\"comment\">{alert_dict['priority']}</font> \n
        {alert_dict['details']}
        """

        payload = {
            "msgtype": "markdown",
            "markdown": {
                "content": content
            }
        }
        resp = requests.request(method='POST',
                                url=self.parent.wecom_webhook_url, 
                                data=json.dumps(payload),
                                headers={'Content-Type': 'application/json'},
                                timeout=3)
        return resp.text


    def send_notify_wecom_card(self, alert_dict):
        if alert_dict['eventType'] == 'trigger':
            source_desc = '告警!'
            icon_url = "https://cdn-icons-png.flaticon.com/128/16750/16750201.png"
            desc_color = 2
        else:
            source_desc = '恢复!'
            icon_url = "https://cdn-icons-png.flaticon.com/128/8888/8888205.png"
            desc_color = 3
            
        card_content = {
            "msgtype":"template_card",
            "template_card":{
                "card_type":"text_notice",
                "source":{
                    "icon_url": icon_url,
                    "desc": source_desc,
                    "desc_color": desc_color
                },
                "main_title":{
                    "title": alert_dict['alarmContent'],
                    "desc":"待补充"
                },
                "sub_title_text": "待补充",
                "horizontal_content_list":[
                    {
                        "keyname":"alarmName",
                        "value": alert_dict['alarmName']
                    },
                    {
                        "keyname":"entityName",
                        "value": alert_dict['entityName'],
                    },
                    {
                        "keyname":"priority",
                        "value": alert_dict['priority'],
                    },     
                    {
                        "keyname":"details",
                        "value": alert_dict['details'].strip(),
                    },       
                ],
                "jump_list":[
                    {
                        "type":1,
                        "url":"https://work.weixin.qq.com/?from=openApi",
                        "title":"Grafana"
                    },

                ],
                "card_action":{
                    "type":1,
                    "url":"https://work.weixin.qq.com/?from=openApi",
                    "appid":"APPID",
                    "pagepath":"PAGEPATH"
                },
                # "quote_area":{
                #     "type":1,
                #     "url":"https://work.weixin.qq.com/?from=openApi",
                #     "appid":"APPID",
                #     "pagepath":"PAGEPATH",
                #     "title":"引用文本标题",
                #     "quote_text":"Jack：企业微信真的很好用~\nBalian：超级好的一款软件！"
                # },
            }
        }
        payload = json.dumps(card_content)
        response = requests.request(method='POST',
                                     url=self.parent.wecom_webhook_url, 
                                     data=payload,
                                     headers=self.headers,
                                     timeout=3)
        return response.text


    def notify_onealert(self):
        pass
    
    def query_alert_counter(self, alarm_content: str) -> str:
        '''
        _summary_

        Args:
            alarm_content (str): _description_

        Returns:
            str: _description_
        '''
        # query = {"alarm_content": alarm_content}
        # count = mongo.collection.count_documents(query)
        # return self.parent.mongo_client.count_documents(query)
        # return count


        query = {
            "alarm_content": alarm_content,
            'resolved_time': {
                '$exists': True,  # 字段必须存在
            }
        }
        fields = {"_id": 0, 'alarm_time': 1, 'resolved_time': 1}
        results = self.parent.mongo_client.find(query, fields).sort('alarm_time', -1)
        
        alarm_list = []
        for result in results:
            duration_minute = '持续 ' + str(int((result['resolved_time'] - result['alarm_time']).total_seconds() / 60)) + ' 分钟'
            alarm_list.append('触发告警: ' + self.time_convert_timeobj_to_str(timeobj=result['alarm_time']) + ' ' + duration_minute)

        alarm_str = '\n'.join(alarm_list)
        
        alarm_str_display_threshold = 10
        
        if len(alarm_list) > alarm_str_display_threshold:
            # 如果告警超过 10 个
            alarm_counter = alarm_str_display_threshold
            alarm_str = '\n'.join(alarm_list[:alarm_str_display_threshold])
        else:
            # 如果不超过 10 个
            alarm_counter = len(alarm_list)
            alarm_str = '\n'.join(alarm_list)
            
        return '该告警出现过' + str(len(alarm_list)) + f'次\n最近 {alarm_counter} 次告警如下: \n' + alarm_str



from ..endpoint import Endpoint
from nb_log import get_logger

log = get_logger('network')


ALARM_NAME_TRAFFIC = '流量超出设置的阀值'
ALARM_NAME_PORT_DOWN = '端口DOWN'

class NetworkEndpoint(Endpoint):
    
    def ping_unreachable(self, limit: int=3):
        QUERY = f"""select
            "percent_packet_loss", 
            "name", 
            "url", 
            "result_code" 
            from ping 
            where time > now() - 1d 
            group by "url" 
            order by desc 
            limit {limit}"""
        
        log.debug(QUERY)
        
        for results in self.parent.influx_client.query(QUERY):
            has_loss = all(result['percent_packet_loss'] == 100.0 for result in results)
            if has_loss:
                r = results[0]
                url = r['url']
                alarm_content = '{} Ping不可达'.format(r['name'])
                log.info(f'发现告警: [{alarm_content}]')
                self.parent.extensions.tool_check_insert_send_mongo(
                    restore_influx=f"""select "percent_packet_loss" from ping where "url" = '{url}' order by time desc limit {limit}""",
                    url=url,
                    alarm_name='Ping 不可达',
                    entity_name=url,
                    alarm_content=alarm_content,
                    alarm_time=self.parent.extensions.time_get_now_time_mongo(),
                    priority='High',
                    is_notify=True)

        for mongo_item in self.parent.extensions.mongo_query_trigger(alarm_name='Ping 不可达'):
            for results in self.parent.influx_client.query(mongo_item['restore_influx']):
                # print(i)
                all_zero = all(result['percent_packet_loss'] == 0.0 for result in results)
                if all_zero:
                    self.parent.extensions.tool_check_insert_send_mongo(
                        event_type='resolved',
                        mongo_id=mongo_item['_id'],
                        event_id=mongo_item['event_id'],
                        entity_name=mongo_item['entity_name'],
                        alarm_name=mongo_item['alarm_name'],
                        alarm_content=mongo_item['alarm_content'],
                        priority=mongo_item['priority'],
                        is_notify=True)

    def packet_loss(self, name: str=None, last_minute: int=10, packet_loss_percent_trigger: int=0.05):
        
        QUERY = f"""select "{name}", "percent_packet_loss", "url" from ping WHERE time > now() - {last_minute}m group by url"""
            
        log.debug(f'packet loss trigger, 查询语法: [{QUERY}]')
        
        for results in self.parent.influx_client.query(QUERY):        
            count_packet_loss = 0
            for result in results:
                if result['percent_packet_loss'] != 0.0:
                    count_packet_loss += 1
            
            packet_loss_percent = count_packet_loss / len(results)
            
            # log.info(packet_loss_percent)
            if 1.0 > packet_loss_percent > packet_loss_percent_trigger:
                
                try:
                    name = results[0][name]
                except KeyError:
                    log.error(f'获取name失败, results为 {results[0]}')
                    name = 'null'
                url = results[0]['url']
                
                alarm_content = f'{name} {url} Ping 丢包率超阀值'
                self.parent.extensions.tool_check_insert_send_mongo(
                    restore_influx=f"""select "percent_packet_loss" from ping where "url" = '{url}' and time > now() - {last_minute}m""",
                    url=url,
                    alarm_name='Ping 丢包率超阀值',
                    entity_name=url,
                    alarm_content=alarm_content,
                    alarm_time=self.parent.extensions.time_get_now_time_mongo(),
                    priority='Warning',
                    is_notify=True,
                    automate_ts=f'设定的阀值为{packet_loss_percent_trigger* 100:.0f}%, 近{last_minute}分钟丢包率为{packet_loss_percent * 100:.0f}%')
                
        for mongo_item in self.parent.extensions.mongo_query_trigger(alarm_name='Ping 丢包率超阀值'):
            for results in self.parent.influx_client.query(mongo_item['restore_influx']):
                
                count_packet_loss = 0
                for result in results:
                    if result['percent_packet_loss'] == 0.0:
                        count_packet_loss += 1
                        
                packet_loss_percent = count_packet_loss / len(results)
                if packet_loss_percent > packet_loss_percent_trigger:
                    self.parent.extensions.tool_check_insert_send_mongo(
                        event_type='resolved',
                        mongo_id=mongo_item['_id'],
                        event_id=mongo_item['event_id'],
                        entity_name=mongo_item['entity_name'],
                        alarm_name=mongo_item['alarm_name'],
                        alarm_content=mongo_item['alarm_content'],
                        priority=mongo_item['priority'],
                        is_notify=True)

    def port_traffic(self, 
             sysname: str=None, 
             agent_host: str=None, 
             ifname: str=None, 
             last_minute: int=30,
             direction: str='inbound',
             threshold: int=100,
             remark: str=''):
        
        if sysname:
            query_property = 'sysName'
            query_value = sysname
            
        
        if agent_host:
            query_property = 'agent_host'
            query_value = agent_host
        
        if direction == 'inbound':
            direction = 'ifHCInOctets'
        elif direction == 'outbound':
            direction = 'ifHCOutOctets'
        
        query = f"""
    select mean(rate) 
    from 
    (SELECT derivative(mean("{direction}"), 1s) * 8 / 1000000 as "rate"
    FROM "interface" WHERE ("{query_property}" = '{query_value}') 
    AND ("ifName" = '{ifname}') 
    AND time > now() - {last_minute}m 
    GROUP BY time(1s), "ifName" fill(null))
    """
    
        log.debug(f'query: [{query.strip()}]')
        for results in self.parent.influx_client.query(query=query):
            # diff_now = self.parent.extensions.time_diff_influx_now(source_time=results['time'])
            log.debug(results)
            traffic_speed = int(results[0]['mean'])
            
            if traffic_speed > threshold:
                
                restore_influx = query.replace('as ', f' - {threshold} as ') + '分隔线' + query
                log.info(f'restore_influx: [{restore_influx}]')
                self.parent.extensions.tool_check_insert_send_mongo(
                    restore_influx=restore_influx,
                    alarm_content= f'{remark}{ALARM_NAME_TRAFFIC},阀值为{threshold}Mbit/s',
                    alarm_name=ALARM_NAME_TRAFFIC,
                    priority='中',
                    entity_name=query_value,
                    alarm_time=self.parent.extensions.time_get_now_time_mongo(),
                    automate_ts=f'当前流量值为{traffic_speed}Mbit/s',
                    is_notify=True)

            
        for mongo_item in self.parent.extensions.mongo_query_trigger(alarm_name=ALARM_NAME_TRAFFIC):
            log.debug('恢复语法: ' + mongo_item['restore_influx'])
            
            all_query = mongo_item['restore_influx'].replace('\\n', '')
            query, current_traffic = all_query.split('分隔线')[0], all_query.split('分隔线')[1]
            log.debug(query)
            for results in self.parent.influx_client.query(query):
                
                log.debug(results)
                
                if results[0]['mean'] < 0:
                    self.parent.extensions.tool_check_insert_send_mongo(
                        mongo_id = mongo_item['_id'],
                        event_type='resolved',
                        alarm_content= mongo_item['alarm_content'],
                        alarm_name=ALARM_NAME_TRAFFIC,
                        priority=mongo_item['priority'],
                        alarm_time=self.parent.extensions.time_get_now_time_mongo(),
                        entity_name=mongo_item['entity_name'],
                        automate_ts=f'当前流量值为{traffic_speed}Mbit/s',
                        is_notify=True)
    
    def port_down(self, 
                sysname: str=None, 
                agent_host: str=None, 
                ifdescr: str=None, 
                remark: str=''):
        query = f"""
        select "sysName", last("ifOperStatus") as ifOperStatus, "ifDescr" 
from interface 
where "sysName" = '{sysname}' and "ifDescr" = '{ifdescr}' and time > now() - 1d"""

        for results in self.parent.influx_client.query(query=query):
            # log.debug(results)
            alarm_content = results[0]['sysName'] + ' ' + results[0]['ifDescr'] + ' 端口DOWN'
            
            if results[0]['ifOperStatus'] != 'up':
                self.parent.extensions.tool_check_insert_send_mongo(
                    restore_influx=f"""select "sysName", last("ifOperStatus") as ifOperStatus, "ifDescr" from interface where "sysName" = '{sysname}' and "ifDescr" = '{ifdescr}'""",
                    alarm_content= alarm_content,
                    alarm_name=ALARM_NAME_PORT_DOWN,
                    priority='中',
                    entity_name=results[0]['sysName'],
                    alarm_time=self.parent.extensions.time_get_now_time_mongo(),
                    automate_ts='',
                    is_notify=True)
                
        # 恢复告警
        for mongo_item in self.parent.extensions.mongo_query_trigger(alarm_name=ALARM_NAME_PORT_DOWN):
            log.debug('恢复语法: ' + mongo_item['restore_influx'])
            for results in self.parent.influx_client.query(mongo_item['restore_influx']):
                
                log.debug(results)
                if results[0]['ifOperStatus'] == 'up':
                    self.parent.extensions.tool_check_insert_send_mongo(
                        mongo_id = mongo_item['_id'],
                        event_type='resolved',
                        alarm_content= mongo_item['alarm_content'] + '已恢复',
                        alarm_name=ALARM_NAME_PORT_DOWN,
                        priority=mongo_item['priority'],
                        alarm_time=self.parent.extensions.time_get_now_time_mongo(),
                        entity_name=mongo_item['entity_name'],
                        automate_ts=f'',
                        is_notify=True)


from ..endpoint import Endpoint
from nb_log import get_logger

log = get_logger('no_data')


ALARM_NAME_NO_DATA = '无监控数据'
ALARM_NAME_UPS_BATTERY_NEEDS_REPLACING = 'UPS 电池需更换'

class CommonEndpoint(Endpoint):
    
    def no_data_node_exporter(self):
        query = """select
    *
    from node_uname_info 
    where time > now() - 1d 
    group by "url"
    order by desc
    limit 1"""
        
        log.debug(f'query: [{query.strip()}]')
        for results in self.parent.influx_client.query(query=query):
            # print(results)
            diff_now = self.parent.extensions.time_diff_influx_now(source_time=results[0]['time'])
            log.debug(diff_now)

            
            if diff_now > 3:
                # print(results)
                nodename = results[0]['nodename']
                
                for url_dict in self.parent.influx_client.query(f"""select * from node_uname_info where "nodename" = '{nodename}' order by time desc limit 1"""):
                    # print(url_dict)
                    url = url_dict[0]['url']
                
                self.parent.extensions.tool_check_insert_send_mongo(
                    restore_influx=f"""select last(*) from node_uname_info where "url" = '{url}'""",
                    alarm_content= results[0]['nodename'] + ' ' + url.replace('http://', '').replace(':9100/metrics', '') + ' ' + ALARM_NAME_NO_DATA,
                    alarm_name=ALARM_NAME_NO_DATA,
                    priority='高',
                    alarm_time=self.parent.extensions.time_get_now_time_mongo(),
                    entity_name=nodename,
                    is_notify=True)


        for mongo_item in self.parent.extensions.mongo_query_trigger(alarm_name=ALARM_NAME_NO_DATA):
            log.debug('恢复语法: ' + mongo_item['restore_influx'])
            
            for results in self.parent.influx_client.query(mongo_item['restore_influx']):
                log.debug(results)
                
                diff_now = self.parent.extensions.time_diff_influx_now(source_time=results[0]['time'])
                nodename = mongo_item['entity_name']
                
                if diff_now < 3:
                    self.parent.extensions.tool_check_insert_send_mongo(mongo_id = mongo_item['_id'],
                        event_type='trigger',
                        alarm_content= mongo_item['alarm_content'],
                        alarm_name=ALARM_NAME_NO_DATA,
                        priority='高',
                        entity_name=nodename,
                        is_notify=True)
                    
    def no_data_windows_exporter(self):
        query = """select
*
from windows_cs_hostname 
where time > now() - 1d 
group by "url"
order by desc
limit 1"""
        
        log.debug(f'query: [{query.strip()}]')
        for results in self.parent.influx_client.query(query=query):
            hostname = results[0]['hostname']
            diff_now = self.parent.extensions.time_diff_influx_now(source_time=results[0]['time'])
            log.debug(f'{hostname} {diff_now}')
            if diff_now > 5:
                
                
                for url_dict in self.parent.influx_client.query(f"""select * from windows_cs_hostname where "hostname" = '{hostname}' order by time desc limit 1"""):
                    url = url_dict['url']
                
                self.parent.extensions.tool_check_insert_send_mongo(
                    restore_influx=f"""select last(*) from windows_cs_hostname where "url" = '{url}'""",
                    alarm_content= results['hostname'] + ' ' + url.replace('http://', '').replace(':9182/metrics', '') + ' ' + ALARM_NAME_NO_DATA,
                    alarm_name=ALARM_NAME_NO_DATA,
                    priority='高',
                    entity_name=hostname,
                    is_notify=True)

        for mongo_item in self.parent.extensions.mongo_query_trigger(alarm_name=ALARM_NAME_NO_DATA):
            log.debug('恢复语法: ' + mongo_item['restore_influx'])
            
            for results in self.parent.influx_client.query(mongo_item['restore_influx']):
                log.debug(results)
                
                diff_now = self.parent.extensions.time_diff_influx_now(source_time=results[0]['time'])
                hostname = mongo_item['entity_name']
                
                if diff_now < 3:
                    self.parent.extensions.tool_check_insert_send_mongo(
                        mongo_id = mongo_item['_id'],
                        event_type='resolved',
                        alarm_content= mongo_item['alarm_content'],
                        alarm_name=ALARM_NAME_NO_DATA,
                        priority='高',
                        entity_name=hostname,
                        is_notify=True)
    
    def not_resolved_trigger(self):
        query = {
            'resolved_time': {'$exists': False}
        } 
        for results in self.parent.mongo_client.find(query):
            print(results)
    
    def ups_apc_advbattery_replace_indicator(self, suggestion):
    #     query= """
    # SELECT "sysName", "upsAdvBatteryReplaceIndicator"  FROM "ups" WHERE time > now() - 10m group by "sysName" order by time desc limit 5
    # """
        query = """
    SELECT "sysName", last("upsAdvBatteryReplaceIndicator")  FROM "ups" WHERE "upsAdvBatteryReplaceIndicator" = 'batteryNeedsReplacing' group by "sysName"
        """
        for results in self.parent.influx_client.query(query=query):
            log.debug(results)
            # has_loss = all(result['upsAdvBatteryReplaceIndicator'] == 'batteryNeedsReplacing' for result in results)
            # if has_loss:
            if results[0]['last']:
            #     log.debug(results)
                sysname = results[0]['sysName']
                self.parent.extensions.tool_check_insert_send_mongo(
                    restore_influx=f"""SELECT last("upsAdvBatteryReplaceIndicator")  FROM "ups" WHERE "sysName" = '{sysname}'""",
                    url=sysname,
                    alarm_name=ALARM_NAME_UPS_BATTERY_NEEDS_REPLACING,
                    entity_name=sysname,
                    alarm_content=f'{sysname} {ALARM_NAME_UPS_BATTERY_NEEDS_REPLACING}',
                    alarm_time=self.parent.extensions.time_get_now_time_mongo(),
                    priority='warning',
                    is_notify=True,
                    suggestion=suggestion)
                
    



from typing import Any, TYPE_CHECKING
if TYPE_CHECKING:
    from .client import BaseClient


class Endpoint:
    
    def __init__(self, parent: "BaseClient") -> None:
        self.parent = parent
    


    
    


#!/usr/bin/env python
# coding: utf-8
from setuptools import setup, find_packages
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


VERSION = '0.1.2'

setup(
    name='InfluxAlert',  # package name
    version=VERSION,  # package version
    description='influx alert',  # package description
    packages=find_packages(),
    url="https://github.com/houm01/InfluxAlert",
    zip_safe=False,
    long_description=long_description,
    long_description_content_type='text/markdown'
)




from .endpoints.network import NetworkEndpoint
from .endpoints.vmware import VMWareEndpoint
from .endpoints.common import CommonEndpoint
from .endpoints.extensions import ExtensionsEndpoint
from influxdb import InfluxDBClient
import urllib3
urllib3.disable_warnings()
from pymongo import MongoClient




class BaseClient:
    
    def __init__(self, 
                 influx_host: str=None, 
                 influx_port: int=8086,
                 influx_username: str=None,
                 influx_password: str=None,
                 influx_database: str='telegraf',
                 influx_ssl: bool=True,
                 mongo_host: str='127.0.0.1',
                 mongo_port: int=27017,
                 mongo_username: str=None,
                 mongo_password: str=None,
                 mongo_authsource: str='admin',
                 mongo_database: str='automate',
                 feishu_app_id: str=None,
                 feishu_app_secret: str=None,
                 feishu_card_template_id: str=None,
                 feishu_card_receive_id: str=None,
                 wecom_webhook_url: str=None,
                 onealert_webhook_url: str=None,
                 debug: bool=False):
        
        self._influx_host = influx_host
        self._influx_port = influx_port
        self._influx_username = influx_username
        self._influx_password = influx_password
        self._influx_database = influx_database
        self._influx_ssl = influx_ssl

        self._mongo_host = mongo_host
        self._mongo_port = mongo_port
        self._mongo_username = mongo_username
        self._mongo_password = mongo_password
        self._mongo_authsource = mongo_authsource
        self._mongo_database = mongo_database


        self.feishu_app_id = feishu_app_id
        self.feishu_app_secret = feishu_app_secret
        self.feishu_card_template_id = feishu_card_template_id
        # self.feishu_card_template_variable = feishu_card_template_variable
        self.feishu_card_receive_id = feishu_card_receive_id
        
        self.wecom_webhook_url = wecom_webhook_url
        self.onealert_webhook_url = onealert_webhook_url
        
        self.debug = debug
        
        self.influx_client = self._build_influxdb_client()
        self.mongo_client = self._build_mongo_client()
        self.feishu_client = self._build_feishu_client()

        self.network = NetworkEndpoint(self)
        self.vmware = VMWareEndpoint(self)
        self.extensions = ExtensionsEndpoint(self)
        self.common = CommonEndpoint(self)
        

    def _build_influxdb_client(self):
        pass

    def _build_mongo_client(self):
        pass
    
    def _build_feishu_client(self):
        pass

class Client(BaseClient):
    def _build_influxdb_client(self):
        return InfluxDBClient(host=self._influx_host, 
                        port=self._influx_port,
                        username=self._influx_username,
                        password=self._influx_password,
                        database=self._influx_database,
                        ssl=self._influx_ssl)
        
    def _build_mongo_client(self):
        client = MongoClient(host=self._mongo_host,
                             port=self._mongo_port,
                             username=self._mongo_username,
                             password=self._mongo_password,
                             authSource=self._mongo_authsource)
        if self.debug:
            return client[self._mongo_database]['alert_debug'] 
        else:
            print(self._mongo_database)
            return client[self._mongo_database]['alert']
    
    def _build_feishu_client(self):
        if self.feishu_app_id:
            from cc_feishu.client import Client as FeishuClient
            return FeishuClient(app_id=self.feishu_app_id, app_secret=self.feishu_app_secret)
    


from .ping_alert_strategy import PingAlertStrategy
from .vmware_cpu_alert_strategy import VMwareCpuAlertStrategy


class InfluxAlert:
    def __init__(self, influx_auth, webhook_url, mongodb_auth):
        self.influx_auth = influx_auth
        self.webhook_url = webhook_url
        self.mongodb_auth = mongodb_auth

        # 初始化告警策略
        self.ping = PingAlertStrategy(influx_auth, webhook_url)
        self.vmware = VMwareCpuAlertStrategy(influx_auth, webhook_url, mongodb_auth)
        
        

if __name__ == '__main__':
    main()